# -*- coding: utf-8 -*-
"""
This class is aim to install lamp server(Linux, Apache, Mysql and PHP) into your
local environment.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s lamp -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao

Contact: zhexiao27@gmail.com
    
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerAbstract import ServerAbstract

# fabric libs
from fabric.colors import red, green
from fabric.api import env, prompt, run, sudo, local, put
from fabric.contrib.files import exists
from fabric.state import output

# hide exec command
output['running'] = False

class ServerLamp(ServerAbstract):
    def __init__(self, args):
        self.args = args

    def install(self):
        self.update_sys()
        self.install_apache()
        self.install_mysql()
        self.install_php()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            sudo('apt-get update -y')

            print(green(' * Done'))
            print()

    def install_apache(self):
        if self.args.force or prompt(red(' * Install apache2 (y/n)?'), default='y') == 'y':
            sudo('apt-get install apache2 -y')

            print(green(' * Done'))
            print()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install mysql (y/n)?'), default='y') == 'y':
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password password {0}'".format(self.mysql_password))
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password_again password {0}'".format(self.mysql_password))
            sudo('apt-get install mysql-server php5-mysql -y')

            print(green(' * Done'))
            print()

    def install_php(self):
        if self.args.force or prompt(red(' * Install php5 (y/n)?'), default='y') == 'y':
            sudo('apt-get install php5 php5-cli libapache2-mod-php5 php5-mcrypt -y')

            # do apache config
            put(StringIO(self.apache_dir_index), '/etc/apache2/mods-enabled/dir.conf', use_sudo=True)

            # write phpinfo for test 
            put(StringIO(self.phpinfo), '{0}/info.php'.format(self.apache_web_dir), use_sudo=True)

            print(green(' * Done'))
            print()




"""
This test is for command line args 
"""

import unittest
from ezhost.ServerLists import ServerLists

class ServerListTest(unittest.TestCase):
    def test_server_exist(self):
        self.assertEqual(ServerLists('lamp').value, 'lamp')

    def test_server_non_exist(self):
        self.assertRaises (ValueError, lambda: ServerLists('namp').value)

if __name__ == '__main__':
    unittest.main()


"""
This test is for config file
"""

import unittest, os
import configparser

class ConfTest(unittest.TestCase):
    def _setup_(self):
        self.config = configparser.ConfigParser()
        self.path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        self.config.read('{0}/data/test_config.ini'.format(self.path))

    def test_config_non_namespace(self):
        self._setup_()

        self.assertNotIn('python', self.config)

    def test_config_namespace(self):
        self._setup_()

        self.assertIn('ezhost', self.config)

    def test_config_value(self):
        self._setup_()
        
        self.assertNotEqual(self.config['ezhost']['user'], 'zx')

    def test_config_non_exist(self):
        self._setup_()

        self.assertRaises (KeyError, lambda: self.config['ezhost']['abc'])

if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
"""
Base class for all server class
"""
from fabric.api import env

from ezhost.ServerAbstract import ServerAbstract
from ezhost.ServerLists import ServerLists
from ezhost.ServerLamp import ServerLamp
from ezhost.ServerLnmp import ServerLnmp
from ezhost.ServerLnmpWordpress import ServerLnmpWordpress
from ezhost.ServerDjangoUwsgi import ServerDjangoUwsgi
from ezhost.ServerCommand import ServerCommand

class ServerBase(ServerAbstract):
    def __init__(self, args, configure_obj):
        self.args = args
        self.configure_obj = configure_obj

        # set common parameters
        self.server_type = self.args.server
        
        # parser parameters
        self._parse_parameters()

        # initial host 
        self.init_host()

        # install 
        self.install()

    def _parse_parameters(self):
        # set args from commmand parser
        if self.configure_obj is None:
            self.host_string = self.args.host
            self.host_user = self.args.user
            self.host_passwd = self.args.passwd
            self.host_keyfile = self.args.keyfile

        # set args from config file parser
        else:
            try:
                self.host_string = self.configure_obj['host']
                self.host_user = self.configure_obj['user']
            except Exception as e:
                raise KeyError('Lack of required host information. Please check whether you have set host address or login user.')
           
            try:
                self.host_keyfile = self.configure_obj['keyfile']
            except Exception as e:
                self.host_keyfile = None

            try:
                self.host_passwd = self.configure_obj['passwd']
            except Exception as e:
                self.host_passwd = None

            if self.host_passwd is None and self.host_keyfile is None:
                raise KeyError('Lack of required host information. Please check whether you have set login password or keyfile.')

    def init_host(self):
        """
            Initial  host 
        """
        env.host_string = self.host_string
        env.user = self.host_user
        env.password = self.host_passwd
        env.key_filename = self.host_keyfile

    def install(self):
        """
            install the server
        """
        try:
            if self.args.server is not None:
                server = ServerLists(self.server_type)
                eval(server.name)(self.args).install()
            else:
                ServerCommand(self.args)
        except Exception as e:
            raise e
       


#!/usr/bin/python
"""
This module contains `main` method plus related subroutines.
`main` is executed as the command line ``ezhost`` program and takes care of
parsing options and commands

The other callables defined in this module are internal only. Anything useful
to individuals leveraging Fabric as a library, should be kept elsewhere.
"""
import argparse
import configparser
from ezhost.ServerBase import ServerBase


def main():
    """
        Check args
    """

    configure_obj = None
    try:
        parser = argparse.ArgumentParser(description='Easy to install server.')
        parser.add_argument(
            '-s', '--server',
            help='what kind of server you want to install',
        )

        # force to install packages without ask question
        parser.add_argument(
            '-f', '--force',
            dest='force',
            action='store_true',
            help='force to install packages without ask question',
        )
        parser.add_argument(
            '-nf', '--not-force',
            dest='force',
            action='store_false',
            help='install packages with ask question',
        )
        parser.set_defaults(force=False)

        # login to remote mysql, default is false
        parser.add_argument(
            '-my', '--mysql',
            dest='login_mysql',
            action='store_true',
            help='login to remote mysql server',
        )
        parser.set_defaults(login_mysql=False)

        # login to server
        parser.add_argument(
            '-login', '--login',
            dest='login_server',
            action='store_true',
            help='login to remote server',
        )
        parser.set_defaults(login_server=False)

        parser.add_argument(
            '-p', '--project',
            help='indicate your project name(some servers need this parameter for initial, default=demo)',
            default='demo'
        )

        parser.add_argument(
            '-gp', '--git-pull',
            help='give me your github project directory, we will auto git pull your server code',
        )

        parser.add_argument(
            '-C', '--config',
            help='config file path of your host informations',
        )

        parser.add_argument(
            '-H', '--host',
            help='host server address',
        )

        parser.add_argument(
            '-U', '--user',
            help='host server login user',
        )

        parser.add_argument(
            '-P', '--passwd',
            help='host server login password',
        )

        parser.add_argument(
            '-K', '--keyfile',
            help='host server key file path',
        )

        args = parser.parse_args()
    except Exception as e:
        parser.print_help()

    # if config file and host both not provide, throw a error
    try:
        if args.config is None and args.host is None:
            raise ValueError('You have to setup your host information through -c(--config) or -H(--host)')
    except Exception as e:
        print(e)
        return

    # check user and passwd
    try:
        if args.host is not None:
            if (args.user and (args.passwd or args.keyfile)) is None:
                raise ValueError('Lack of required host information. Please check whether you have set login user, login password or keyfile.')
    except Exception as e:
        print(e)
        return

    # if exist config file, read configuration from file
    try:
        if args.config is not None:
            # init configuration parser
            configure = configparser.ConfigParser()
            configure.read(args.config)

            # check key exist
            if 'ezhost' in configure:
                configure_obj = configure['ezhost']
            else:
                raise KeyError('Can not found ezhost configuration informations.')
    except Exception as e:
        print(e)
        return

    # init server
    ServerBase(args, configure_obj)



# -*- coding: utf-8 -*-
"""
This class is aim to exec all command on the server

Usage:
    $ sudo pip install ezhost
    $ ezhost -gp /var/www/html -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao

Contact: zhexiao27@gmail.com
    
Github: https://github.com/zhexiao/ezhost.git
"""

from ezhost.ServerAbstract import ServerAbstract

# fabric libs
from fabric.colors import red, green
from fabric.api import env, prompt, run, sudo, local, put
from fabric.contrib.files import exists
from fabric.state import output

# hide exec command
output['running'] = False

class ServerCommand(ServerAbstract):
    def __init__(self, args):
        self.args = args

        # parse args
        self.dispatch_args()

    def dispatch_args(self):
        if self.args.git_pull is not None:
            self.git_pull()

        if self.args.login_mysql:
            self.login_mysql()

        if self.args.login_server:
            self.login_server()

    def install(self):
        pass

    def git_pull(self):
        """
        Doing git pull code from github
        """
        run('cd {0} && git pull'.format(self.args.git_pull))

    def login_mysql(self):
        """
        Login to remote mysql server
        """
        run('mysql -u root -p')

    def login_server(self):
        """
        Login to server
        """
        local('ssh -i {0} {1}@{2}'.format(env.key_filename, env.user, env.host_string))


# -*- coding: utf-8 -*-
"""
This class is aim to install wordpress project in lnmp server(Linux, Nginx, Mysql and PHP) into your host.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s lnmp-wordpress -p news -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao

Contact: zhexiao27@gmail.com
    
Github: https://github.com/zhexiao/ezhost.git
"""
from io import StringIO

from ezhost.ServerAbstract import ServerAbstract
from ezhost.ServerLnmp import ServerLnmp

# fabric libs
from fabric.colors import red, green
from fabric.api import env, prompt, run, sudo, local, put
from fabric.contrib.files import exists, sed
from fabric.state import output
from fabric.context_managers import cd

# hide exec command
output['running'] = False


class ServerLnmpWordpress(ServerAbstract):

    def __init__(self, args):
        self.args = args
        self.project = self.args.project

    def install(self):
        # first install lnmp server
        ServerLnmp(self.args).install()

        self.install_wordpress()
        self.vagrant_workspace()

    def install_wordpress(self):
        if self.args.force or prompt(red(' * Install wordpress (y/n)?'), default='y') == 'y':
            # These two packages allow you to work with images and
            # install/update plugins and components using SSH respectively.
            sudo('sudo apt-get install php5-gd libssh2-php -y')

            # go to web root
            with cd(self.nginx_web_dir):
                # download latest wordpress   
                sudo('wget https://wordpress.org/latest.tar.gz')
                sudo('tar -zxvf latest.tar.gz')
                sudo('sudo chown -R www-data:www-data wordpress')
                # rename wordpress project
                sudo('mv wordpress {0}'.format(self.project))
                # create uploads folder
                sudo('mkdir {0}/wp-content/uploads'.format(self.project))
                sudo('chown -R www-data:www-data {0}/wp-content/uploads'.format(self.project))

            # go to nginx available config
            with cd('/etc/nginx/sites-available'):
                if not exists(self.project):
                    sudo('touch {0}'.format(self.project))
                put(StringIO(self.nginx_web_wordpress_config), self.project, use_sudo=True)

            # go to nginx enabled config
            with cd('/etc/nginx/sites-enabled'):
                if exists(self.project):
                    sudo('rm {0}'.format(self.project))

                # remove the default nginx configuration
                if exists('default'):
                    sudo('rm default')
                    
                sudo('ln -s /etc/nginx/sites-available/{0} .'.format(self.project))

            # restart server
            sudo('service nginx restart')
            sudo('service php5-fpm restart')

            print(green(' * Done'))
            print()

    def vagrant_workspace(self):
        if prompt(red(' * Are you working on the vagrant server (y/n)?'), default='y') == 'y':
            sudo('mv /var/www/html/ /vagrant/')
            sudo('cd /var/www/ && ln -s /vagrant/html/ .')



from enum import Enum

"""
All valid server lists
"""
class ServerLists(Enum):
    ServerLamp = 'lamp'
    ServerLnmp = 'lnmp'
    ServerLnmpWordpress = 'lnmp-wordpress'
    ServerDjangoUwsgi = 'django-uwsgi'


# -*- coding: utf-8 -*-
"""
This class is aim to install Django Applications with uWSGI and Nginx, 
we will using linux supervisor to control the server auto start.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s django-uwsgi -H 127.0.0.1:2200 -U vagrant -P vagrant

Author: Zhe Xiao

Contact: zhexiao27@gmail.com
    
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerAbstract import ServerAbstract

# fabric libs
from fabric.colors import red, green
from fabric.api import env, prompt, run, sudo, local, put
from fabric.contrib.files import exists
from fabric.state import output

# hide exec command
output['running'] = False

class ServerDjangoUwsgi(ServerAbstract):
    def __init__(self, args):
        self.args = args
        self.project = self.args.project
        self.project_dir = '{0}/{1}'.format(self.nginx_web_dir, self.project)

    def install(self):
        self.update_sys()
        self.install_mysql()
        self.install_libraries()
        self.install_project()
        self.install_nginx()
        self.install_uwsgi()
        self.install_supervisor()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            sudo('apt-get update -y')

            print(green(' * successfully updated your system package'))
            print()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install mysql (y/n)?'), default='y') == 'y':
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password password {0}'".format(self.mysql_password))
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password_again password {0}'".format(self.mysql_password))
            sudo('apt-get install mysql-server -y')

            print(green(' * Done'))
            print()

    def install_libraries(self):
        if self.args.force or prompt(red(' * Install libraries (y/n)?'), default='y') == 'y':
            sudo('apt-get install python3 python3-pip python-pip -y')
            sudo('pip install virtualenv')

            print(green(' * Done '))
            print()

    def install_project(self):
        if self.args.force or prompt(red(' * Install Django project (y/n)?'), default='y') == 'y':
            # create project
            if not exists( self.project_dir ):
                sudo('mkdir -p {0}'.format(self.project_dir) )
            sudo('chmod -R 777 {0}'.format(self.project_dir) )

            # create project virtualenv
            run('virtualenv -p python3 {0}'.format(self.python_env_dir) )
            run('{0}/bin/pip install django markdown django-filter'.format(self.python_env_dir) )
            run('cd {0} && {1}/bin/django-admin startproject {2} .'.format(self.project_dir, self.python_env_dir, self.project))

            print(green(' * Done '))
            print()

    def install_nginx(self):
        if self.args.force or prompt(red(' * Install Nginx server (y/n)?'), default='y') == 'y':
            sudo('apt-get install nginx -y')

            # nginx configuration
            put(StringIO(self.django_uwsgi_with_nginx), '/etc/nginx/sites-enabled/default', use_sudo=True)

            # restart server and supervisor
            sudo('service nginx restart')

            print(green(' * Done '))
            print()

    def install_uwsgi(self):
        if self.args.force or prompt(red(' * Install Uwsgi service (y/n)?'), default='y') == 'y':
            sudo('pip3 install uwsgi')
            
            # create uwsgi ini file
            if not exists('{0}/{1}.ini'.format(self.project_dir, self.project)):
                run('touch {0}/{1}.ini'.format(self.project_dir, self.project))

            # uwsgi configuration
            django_uwsgi_ini = self.django_uwsgi_ini.format(self.nginx_web_dir, self.project, self.python_env_dir)
            put(StringIO(django_uwsgi_ini), '{0}/{1}.ini'.format(self.project_dir, self.project), use_sudo=True)

            print(green(' * Done '))
            print()

    def install_supervisor(self):
        if self.args.force or prompt(red(' * Install Supervisor for control Uwsgi service (y/n)?'), default='y') == 'y':
            sudo('apt-get install supervisor -y')

            # create supervisor control configruation
            if not exists('{0}/{1}_sysd.conf'.format(self.supervisor_config_dir, self.project) ):
                sudo('touch {0}/{1}_sysd.conf'.format(self.supervisor_config_dir, self.project) )

            # create supervisor log and error file
            if not exists('/var/log/{0}_out.log'.format(self.project) ):
                sudo('touch /var/log/{0}_out.log'.format(self.project) )
            if not exists('/var/log/{0}_error.log'.format(self.project) ):
                sudo('touch /var/log/{0}_error.log'.format(self.project) )

            # supervisor control uwsgi config
            supervisor_uwsgi_ini = self.supervisor_uwsgi_ini.format(self.project, self.project_dir)
            put(StringIO(supervisor_uwsgi_ini), '{0}/{1}_sysd.conf'.format(self.supervisor_config_dir, self.project) , use_sudo=True)

            sudo('sudo supervisorctl reread && sudo supervisorctl update')

            print(green(' * Done '))
            print()


# -*- coding: utf-8 -*-
"""
Abstract class for all server class
"""

# internal libs
import os
from abc import ABCMeta, abstractmethod, abstractproperty

class ServerAbstract(metaclass=ABCMeta):
    def __init__(self):
        # host information
        self._host_string = None
        self._host_user = None
        self._host_passwd = None
        self._host_keyfile = None
        # server type
        self._server_type = None
        self._project = None

    @property
    def host_string(self):
        """
            Host String getter
        """
        return self._host_string

    @host_string.setter
    def host_string(self, value):
        """
            Host string setter
        """
        self._host_string = value

    @property
    def host_user(self):
        """
            Host user getter
        """
        return self._host_user

    @host_user.setter
    def host_user(self, value):
        """
            Host user setter
        """
        self._host_user = value

    @property
    def host_passwd(self):
        """
            Host password getter
        """
        return self._host_passwd

    @host_passwd.setter
    def host_passwd(self, value):
        """
            Host password setter
        """
        self._host_passwd = value

    @property
    def host_keyfile(self):
        """
            Host keyfile getter
        """
        return self._host_keyfile

    @host_keyfile.setter
    def host_keyfile(self, value):
        """
            Host keyfile setter
        """
        self._host_keyfile = value

    @property
    def server_type(self):
        """
            Server type getter
        """
        return self._server_type

    @server_type.setter
    def server_type(self, value):
        """
            Server type setter
        """
        self._server_type = value
    
    @property
    def project(self):
        """
            project getter
        """
        return self._project

    @project.setter
    def project(self, value):
        """
            project setter
        """
        self._project = value

    @property
    def command_path(self):
        """
            Currently command path
        """
        return os.path.dirname(os.path.realpath(__file__))

    @property
    def mysql_password(self):
        """
            Mysql password
        """
        return 'password'

    @property
    def apache_web_dir(self):
        """
            default apache project dir
        """
        return '/var/www/html'

    @property
    def nginx_web_dir(self):
        """
            default apache project dir
        """
        return '/var/www/html'

    @property
    def apache_dir_index(self):
        """
            Currently, if a user requests a directory from the server, Apache 
            will first look for a file called index.html. We want to tell our 
            web server to prefer PHP files, so we'll make Apache look for an 
            index.php file first.
        """
        long_text = """<IfModule mod_dir.c>
    DirectoryIndex index.php index.html index.cgi index.pl index.xhtml index.htm
</IfModule>
        """
        return long_text

    @property
    def phpinfo(self):
        """
           In order to test that our system is configured properly for PHP, 
           we can create a very basic PHP script.
        """
        long_text = """<?php
phpinfo();
?>
        """
        return long_text

    @property
    def nginx_web_config(self):
        """
           Nginx web config
        """
        long_text = """server 
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;   
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
    }}
}}
        """

        long_text = long_text.format(self.nginx_web_dir)
        return long_text

    @property
    def nginx_web_ssl_config(self):
        """
           Nginx web ssl config
        """
        long_text = """server 
{{
    listen 443 default_server;
    listen [::]:443 default_server;

    root {dt[0]};
    autoindex on;
    
    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    ssl on;
    ssl_certificate {dt[1]}/cert.pem;
    ssl_certificate_key {dt[1]}/cert.key;

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;   
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
    }}
}}
        """

        config = [self.nginx_web_dir, self.nginx_ssl_dir]
        long_text = long_text.format(dt=config)
        return long_text

    @property
    def python_env_dir(self):
        """
            Python virtualenv dir
        """
        return '{0}/{1}/env'.format(self.nginx_web_dir, self.project)

    @property
    def django_uwsgi_ini(self):
        """
           This ini file is django uwsgi configuration
        """
        long_text = """[uwsgi]
chdir = {0}/{1}
home = {2}
module = {1}.wsgi:application

uid = vagrant
gid = www-data

master = true
processes = 5

socket = /tmp/{1}.sock
chmod-socket = 664
vacuum = true
        """
        return long_text

    @property
    def django_uwsgi_with_nginx(self):
        """
           This ini file nginx configuration with uwsgi application
        """
        long_text = """server {{
    listen 80;
    server_name localhost;

    location = /favicon.ico {{ 
        access_log off; 
        log_not_found off; 
    }}

    location / {{
        include uwsgi_params;
        uwsgi_pass unix:/tmp/{0}.sock;
    }}
}}
        """
        long_text = long_text.format(self.project)
        return long_text

    @property
    def supervisor_uwsgi_ini(self):
        """
           supervisor control uwsgi configuration 
        """
        long_text = """[program:{0}]
command=uwsgi --ini {1}/{0}.ini
directory={1}
numprocs=1
stdout_logfile=/var/log/{0}_out.log
stderr_logfile=/var/log/{0}_error.log
autostart=true
autorestart=true
startsecs=2
stopwaitsecs=2
killasgroup=true
priority=998   
        """
        return long_text

    @property
    def supervisor_config_dir(self):
        """
            supervisor control config dir
        """
        return '/etc/supervisor/conf.d'


    @property
    def nginx_ssl_dir(self):
        """
            ssl directory
        """
        return '/etc/nginx/ssl'


    @property
    def nginx_web_wordpress_config(self):
        """
           Nginx web wordpress config
        """
        long_text = """server 
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0}/{1};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;
    location / {{
        try_files $uri $uri/ /index.php?q=$uri&$args;
    }}

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;   
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
       fastcgi_split_path_info ^(.+\.php)(/.+)$;
    }}
}}
        """

        long_text = long_text.format(self.nginx_web_dir, self.project)
        return long_text

    @abstractmethod
    def install(self):
        """
            All subclass need implement this function
        """
        pass



# -*- coding: utf-8 -*-
"""
This class is aim to install lnmp server(Linux, Nginx, Mysql and PHP) into your
local environment.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s lnmp -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao

Contact: zhexiao27@gmail.com
    
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerAbstract import ServerAbstract

# fabric libs
from fabric.colors import red, green
from fabric.api import env, prompt, run, sudo, local, put
from fabric.contrib.files import exists, sed
from fabric.state import output

# hide exec command
output['running'] = False

class ServerLnmp(ServerAbstract):
    def __init__(self, args):
        self.args = args

    def install(self):
        self.update_sys()
        self.install_mysql()
        self.install_nginx()
        self.install_php()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            sudo('apt-get update -y')

            print(green(' * Done'))
            print()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install mysql (y/n)?'), default='y') == 'y':
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password password {0}'".format(self.mysql_password))
            sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password_again password {0}'".format(self.mysql_password))
            sudo('apt-get install mysql-server php5-mysql -y')

            print(green(' * Done'))
            print()

    def install_nginx(self):
        if self.args.force or prompt(red(' * Install Nginx (y/n)?'), default='y') == 'y':
            run('echo "deb http://ppa.launchpad.net/nginx/stable/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/nginx-stable.list')
            sudo('apt-key adv --keyserver keyserver.ubuntu.com --recv-keys C300EE8C')
            sudo('apt-get update -y')
            sudo('apt-get install nginx -y')

            # do nginx config 
            put(StringIO(self.nginx_web_config), '/etc/nginx/sites-available/default', use_sudo=True)


        # want to using https?
        if prompt(red(' * Change url from http to https (y/n)?'), default='y') == 'y':
            if not exists(self.nginx_ssl_dir):
                sudo('mkdir -p {0}'.format(self.nginx_ssl_dir) )
            # generate ssh key
            sudo('openssl req -x509 -nodes -days 365 -newkey rsa:2048 -keyout {0}/cert.key -out {0}/cert.pem'.format(self.nginx_ssl_dir) )

            # do nginx config config
            put(StringIO(self.nginx_web_ssl_config), '/etc/nginx/sites-available/default', use_sudo=True)

        sudo('service nginx restart')

        print(green(' * Done'))
        print()


    def install_php(self):
        if self.args.force or prompt(red(' * Install php5 (y/n)?'), default='y') == 'y':
            # Find the line, cgi.fix_pathinfo=1, and change the 1 to 0.
            sudo('apt-get install php5-fpm -y')
            sed('/etc/php5/fpm/php.ini', ';cgi.fix_pathinfo=1', 'cgi.fix_pathinfo=0', use_sudo=True)

            # write phpinfo for test 
            put(StringIO(self.phpinfo), '{0}/info.php'.format(self.nginx_web_dir), use_sudo=True)

            sudo('service php5-fpm restart')

            print(green(' * Done'))
            print()

